Package: preciseTAD
Type: Package
Title: preciseTAD: A machine learning framework for precise TAD
        boundary prediction
Version: 1.22.0
Authors@R: c(
    person("Spiro", "Stilianoudakis", 
        email = "stilianoudasc@vcu.edu", 
        role = c("aut")),
    person("Mikhail", "Dozmorov", 
        email = "mikhail.dozmorov@gmail.com", 
        role = c("aut", "cre")))
Description: preciseTAD provides functions to predict the location of boundaries 
    of topologically associated domains (TADs) and chromatin loops at base-level 
    resolution. As an input, it takes BED-formatted genomic coordinates of 
    domain boundaries detected from low-resolution Hi-C data, and coordinates of 
    high-resolution genomic annotations from ENCODE or other consortia. 
    preciseTAD employs several feature engineering strategies and resampling 
    techniques to address class imbalance, and trains an optimized random forest 
    model for predicting low-resolution domain boundaries. Translated on 
    a base-level, preciseTAD predicts the probability for each base to be 
    a boundary. Density-based clustering and scalable partitioning techniques 
    are used to detect precise boundary regions and summit points. Compared with 
    low-resolution boundaries, preciseTAD boundaries are highly enriched for 
    CTCF, RAD21, SMC3, and ZNF143 signal and more conserved across cell lines. 
    The pre-trained model can accurately predict boundaries in another cell line 
    using CTCF, RAD21, SMC3, and ZNF143 annotation data for this cell line. 
License: MIT + file LICENSE
Depends: R (>= 4.1)
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.1.1
Suggests: knitr, rmarkdown, testthat, BiocCheck, BiocManager, BiocStyle
VignetteBuilder: knitr
Imports: S4Vectors, IRanges, GenomicRanges, randomForest, ModelMetrics,
        e1071, PRROC, pROC, caret, utils, cluster, dbscan, doSNOW,
        foreach, pbapply, stats, parallel, gtools, rCGH
biocViews: Software, HiC, Sequencing, Clustering, Classification,
        FunctionalGenomics, FeatureExtraction
BugReports: https://github.com/dozmorovlab/preciseTAD/issues
URL: https://github.com/dozmorovlab/preciseTAD
git_url: https://git.bioconductor.org/packages/preciseTAD
git_branch: RELEASE_3_23
git_last_commit: dda7c03
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:00:14 UTC; biocbuild
Author: Spiro Stilianoudakis [aut],
  Mikhail Dozmorov [aut, cre]
Maintainer: Mikhail Dozmorov <mikhail.dozmorov@gmail.com>
